@@ -1,7 +1,7 @@
 #' Number of dimensions.
 #'
 #' Number of dimensions of an array or vector
-#' 
+#'
 #' @param x array
 #' @keywords internal
 dims <- function(x) length(amv_dim(x))
@@ -9,30 +9,30 @@ dims <- function(x) length(amv_dim(x))
 #' Dimensions.
 #'
 #' Consistent dimensions for vectors, matrices and arrays.
-#' 
+#'
 #' @param x array, matrix or vector
-#' @keywords internal 
+#' @keywords internal
 amv_dim <- function(x) if (is.vector(x)) length(x) else dim(x)
 
 
 #' Dimension names.
 #'
 #' Consistent dimnames for vectors, matrices and arrays.
-#' 
+#'
 #' Unlike \code{\link{dimnames}} no part of the output will ever be
 #' null.  If a component of dimnames is omitted, \code{amv_dimnames}
 #' will return an integer sequence of the appropriate length.
-#' 
+#'
 #' @param x array, matrix or vector
-#' @keywords internal 
+#' @keywords internal
 #' @export
 amv_dimnames <- function(x) {
   d <- if (is.vector(x)) list(names(x)) else dimnames(x)
-  
+
   if (is.null(d)) d <- rep(list(NULL), dims(x))
   null_names <- which(unlist(llply(d, is.null)))
-  d[null_names] <- llply(null_names, function(i) seq.int(amv_dim(x)[i]))
-  
+  d[null_names] <- llply(null_names, function(i) seq_len(amv_dim(x)[i]))
+
   # if (is.null(names(d))) names(d) <- paste("X", 1:length(d), sep="")
   d
 }
@@ -40,10 +40,11 @@ amv_dimnames <- function(x) {
 #' Reduce dimensions.
 #'
 #' Remove extraneous dimensions
-#' 
+#'
 #' @param x array
-#' @keywords internal 
+#' @keywords internal
 reduce_dim <- function(x) {
-  do.call("[", c(list(x), lapply(dim(x), function(x) if (x==1) 1 else TRUE), drop=TRUE))  
+  subs <- lapply(dim(x), function(x) if (x == 1) 1 else bquote())
+  call <- as.call(c(list(as.name("["), quote(x)), subs, list(drop = TRUE)))
+  eval(call)
 }
-
